@@ -1,23 +1,6 @@
-// "$Id: $"
-//
-// FLTK native file chooser widget wrapper for GTK's GtkFileChooserDialog 
-//
-// Copyright 1998-2013 by Bill Spitzak and others.
-// Copyright 2012 IMM
-//
-// This library is free software. Distribution and use rights are outlined in
-// the file "COPYING" which should have been included with this file.  If this
-// file is missing or damaged, see the license at:
-//
-//     http://www.fltk.org/COPYING.php
-//
-// Please report all bugs and problems to:
-//
-//     http://www.fltk.org/str.php
-//
-
-#  include <dlfcn.h>   // for dlopen et al
-#  include <locale.h>  // for setlocale
+#include <FL/x.H>
+#include <dlfcn.h>   // for dlopen et al
+#include <locale.h>  // for setlocale
 
 /* --------------------- Type definitions from GLIB and GTK --------------------- */
 /* all of this is from the public gnome API, so unlikely to change */
@@ -44,6 +27,7 @@ typedef struct _GtkWidget      GtkWidget;
 typedef struct _GtkFileChooser      GtkFileChooser;
 typedef struct _GtkDialog        GtkDialog;
 typedef struct _GtkWindow          GtkWindow;
+typedef struct _GdkDrawable           GdkWindow;
 typedef struct _GtkFileFilter     GtkFileFilter;
 typedef struct _GtkToggleButton       GtkToggleButton;
 typedef enum {
@@ -74,7 +58,9 @@ typedef enum
 #define GTK_STOCK_CANCEL           "gtk-cancel"
 #define GTK_STOCK_SAVE             "gtk-save"
 #define GTK_STOCK_OPEN             "gtk-open"
+const int   GTK_RESPONSE_NONE = -1;
 const int   GTK_RESPONSE_ACCEPT = -3;
+const int   GTK_RESPONSE_DELETE_EVENT = -4;
 const int   GTK_RESPONSE_CANCEL = -6;
 typedef void  (*GCallback)(void);
 #define	G_CALLBACK(f)			 ((GCallback) (f))
@@ -84,7 +70,7 @@ typedef void  (*GClosureNotify)(gpointer data, GClosure	*closure);
 
 /* --------------------- End of Type definitions from GLIB and GTK --------------------- */
 
-int Fl_GTK_File_Chooser::did_find_GTK_libs = 0;
+int mgFl_GTK_File_Chooser::did_find_GTK_libs = 0;
 
 /* These are the GTK/GLib methods we want to load, but not call by name...! */
 
@@ -140,10 +126,6 @@ static XX_gtk_file_chooser_get_select_multiple fl_gtk_file_chooser_get_select_mu
 typedef void (*XX_gtk_widget_hide)(GtkWidget *);
 static XX_gtk_widget_hide fl_gtk_widget_hide = NULL;
 
-// gint gtk_dialog_run(GtkDialog *dialog);
-typedef gint (*XX_gtk_dialog_run)(GtkDialog *dialog);
-static XX_gtk_dialog_run fl_gtk_dialog_run = NULL;
-
 // gchar * gtk_file_chooser_get_filename(GtkFileChooser *chooser);
 typedef gchar* (*XX_gtk_file_chooser_get_filename)(GtkFileChooser *);
 static XX_gtk_file_chooser_get_filename fl_gtk_file_chooser_get_filename = NULL;
@@ -203,9 +185,17 @@ static XX_gtk_file_filter_get_name fl_gtk_file_filter_get_name = NULL;
 typedef void (*XX_gtk_file_chooser_set_extra_widget)(GtkFileChooser *, GtkWidget *);
 static XX_gtk_file_chooser_set_extra_widget fl_gtk_file_chooser_set_extra_widget = NULL;
 
-// void gtk_widget_show(GtkWidget *);
-typedef void (*XX_gtk_widget_show)(GtkWidget *);
-static XX_gtk_widget_show fl_gtk_widget_show = NULL;
+// void gtk_widget_show_now(GtkWidget *);
+typedef void (*XX_gtk_widget_show_now)(GtkWidget *);
+static XX_gtk_widget_show_now fl_gtk_widget_show_now = NULL;
+
+// GdkWindow* gtk_widget_get_window(GtkWidget *);
+typedef GdkWindow* (*XX_gtk_widget_get_window)(GtkWidget *);
+static XX_gtk_widget_get_window fl_gtk_widget_get_window = NULL;
+
+// Window gdk_x11_drawable_get_xid(GdkWindow *);
+typedef Window (*XX_gdk_x11_drawable_get_xid)(GdkWindow *);
+static XX_gdk_x11_drawable_get_xid fl_gdk_x11_drawable_get_xid = NULL;
 
 // GtkWidget *gtk_check_button_new_with_label(const gchar *);
 typedef GtkWidget* (*XX_gtk_check_button_new_with_label)(const gchar *);
@@ -232,7 +222,7 @@ typedef void (*XX_gtk_toggle_button_set_active)(GtkToggleButton *, gboolean);
 static XX_gtk_toggle_button_set_active fl_gtk_toggle_button_set_active = NULL;
 
 
-Fl_GTK_File_Chooser::Fl_GTK_File_Chooser(int val) : Fl_FLTK_File_Chooser(-1)
+mgFl_GTK_File_Chooser::mgFl_GTK_File_Chooser(int val) : mgFl_FLTK_File_Chooser(-1)
 {
   gtkw_ptr   = NULL;    // used to hold a GtkWidget* 
   gtkw_slist = NULL;    // will hold the returned file names in a multi-selection...
@@ -243,7 +233,7 @@ Fl_GTK_File_Chooser::Fl_GTK_File_Chooser(int val) : Fl_FLTK_File_Chooser(-1)
   previous_filter = NULL;
 }
 
-Fl_GTK_File_Chooser::~Fl_GTK_File_Chooser()
+mgFl_GTK_File_Chooser::~mgFl_GTK_File_Chooser()
 {
   // Should free up resources taken for...
   if(gtkw_ptr) { 
@@ -267,15 +257,15 @@ Fl_GTK_File_Chooser::~Fl_GTK_File_Chooser()
   gtkw_title = strfree(gtkw_title);
 }
 
-void Fl_GTK_File_Chooser::type(int val) {
+void mgFl_GTK_File_Chooser::type(int val) {
   _btype = val;
 }
 
-int Fl_GTK_File_Chooser::count() const {
+int mgFl_GTK_File_Chooser::count() const {
   return gtkw_count;
 }
 
-const char *Fl_GTK_File_Chooser::filename() const
+const char *mgFl_GTK_File_Chooser::filename() const
 {
   if(gtkw_ptr) {
     if(fl_gtk_file_chooser_get_select_multiple((GtkFileChooser *)gtkw_ptr) == FALSE) {
@@ -290,7 +280,7 @@ const char *Fl_GTK_File_Chooser::filename() const
   return("");
 }
 
-const char *Fl_GTK_File_Chooser::filename(int i) const
+const char *mgFl_GTK_File_Chooser::filename(int i) const
 {
   if(fl_gtk_file_chooser_get_select_multiple((GtkFileChooser *)gtkw_ptr) == FALSE) {
     return gtkw_filename;
@@ -305,19 +295,19 @@ const char *Fl_GTK_File_Chooser::filename(int i) const
   return("");
 }
 
-void Fl_GTK_File_Chooser::title(const char *val)
+void mgFl_GTK_File_Chooser::title(const char *val)
 {
   strfree(gtkw_title);
   gtkw_title = strnew(val);
 }
 
-const char* Fl_GTK_File_Chooser::title() const
+const char* mgFl_GTK_File_Chooser::title() const
 {
   return gtkw_title;
 }
 
 /* changes the extension of the outfile in the chooser according to newly selected filter */
-void Fl_GTK_File_Chooser::changed_output_type(const char *filter)
+void mgFl_GTK_File_Chooser::changed_output_type(const char *filter)
 {
   if ( !(options()&Fl_Native_File_Chooser::USE_FILTER_EXT) ) return;
   if (strchr(filter, '(') || strchr(filter, '{') || strchr(filter+1, '*') || strncmp(filter, "*.", 2)) return;
@@ -335,7 +325,7 @@ void Fl_GTK_File_Chooser::changed_output_type(const char *filter)
 
 /* Filters files before display in chooser. 
  Also used to detect when the filter just changed */
-gboolean Fl_GTK_File_Chooser::custom_gtk_filter_function(const GtkFileFilterInfo *info, Fl_GTK_File_Chooser::pair* p)
+gboolean mgFl_GTK_File_Chooser::custom_gtk_filter_function(const GtkFileFilterInfo *info, mgFl_GTK_File_Chooser::pair* p)
 {
   if (p->running->previous_filter != p->filter) {
     p->running->changed_output_type(p->filter);
@@ -344,7 +334,7 @@ gboolean Fl_GTK_File_Chooser::custom_gtk_filter_function(const GtkFileFilterInfo
   return (gboolean)fl_filename_match(info->filename, p->filter);
 }
 
-void Fl_GTK_File_Chooser::free_pair(Fl_GTK_File_Chooser::pair *p)
+void mgFl_GTK_File_Chooser::free_pair(mgFl_GTK_File_Chooser::pair *p)
 {
   delete p;
 }
@@ -355,29 +345,45 @@ static void hidden_files_cb(GtkToggleButton *togglebutton, gpointer user_data)
   fl_gtk_file_chooser_set_show_hidden((GtkFileChooser*)user_data, state);
 }
 
-int Fl_GTK_File_Chooser::show()
+int mgFl_GTK_File_Chooser::show()
 {
-  char *p, **before = new char*[20];
-  int val;
-  for (val = 0; val < 20; val++) { // memorize the state of all components of the current locale
-    if (val == LC_ALL) continue;
-    p = setlocale(val, NULL);
-    if (!p) break;
-    before[val] = strdup(p);
+  // The point here is that after running a GTK dialog, the calling program's current locale is modified.
+  // To avoid that, we memorize the calling program's current locale, and the locale as modified
+  // by GTK after the first dialog use. We restore the calling program's current locale 
+  // before returning, and we set the locale as modified by GTK before subsequent GTK dialog uses.
+  static bool first = true;
+  char *p;
+  char *before = NULL;
+  static char *gtk_wants = NULL;
+  // record in before the calling program's current locale
+  p = setlocale(LC_ALL, NULL);
+  if (p) before = strdup(p);
+  if (gtk_wants) { // set the locale as GTK 'wants it'
+    setlocale(LC_ALL, gtk_wants);
   }
   int retval = fl_gtk_chooser_wrapper(); // may change the locale
-  for (int i = 0; i < val; i++) { // set all components of the locale to their previous state
-    if (i == LC_ALL) continue;
-    if (before[i]) {
-      setlocale(i, before[i]);
-      free(before[i]);
-    }
+  if (first) {
+    first = false;
+    // record in gtk_wants the locale as modified by the GTK dialog
+    p = setlocale(LC_ALL, NULL);
+    if (p) gtk_wants = strdup(p);
   }
-  delete[] before;
+  if (before) {
+    setlocale(LC_ALL, before); // restore calling program's current locale
+    free(before);
+    }
   return retval;
 }
 
-int Fl_GTK_File_Chooser::fl_gtk_chooser_wrapper()
+
+static void run_response_handler(GtkDialog *dialog, gint response_id, gpointer data)
+{
+  gint *ri = (gint *)data;
+  *ri = response_id;
+}
+
+
+int mgFl_GTK_File_Chooser::fl_gtk_chooser_wrapper()
 {
   int result = 1;
   static int have_gtk_init = 0;
@@ -465,9 +471,9 @@ int Fl_GTK_File_Chooser::fl_gtk_chooser_wrapper()
       char *q = strchr(p, ')'); *q = 0;
       fl_gtk_file_filter_add_custom(filter_tab[count], 
 				    GTK_FILE_FILTER_FILENAME, 
-				    (GtkFileFilterFunc)Fl_GTK_File_Chooser::custom_gtk_filter_function, 
-				    new Fl_GTK_File_Chooser::pair(this, p), 
-				    (GDestroyNotify)Fl_GTK_File_Chooser::free_pair);
+				    (GtkFileFilterFunc)mgFl_GTK_File_Chooser::custom_gtk_filter_function, 
+				    new mgFl_GTK_File_Chooser::pair(this, p), 
+				    (GDestroyNotify)mgFl_GTK_File_Chooser::free_pair);
       fl_gtk_file_chooser_add_filter((GtkFileChooser *)gtkw_ptr, filter_tab[count]);
       p = strtok(NULL, "\t");
       count++;
@@ -484,14 +490,32 @@ int Fl_GTK_File_Chooser::fl_gtk_chooser_wrapper()
   }
   
   GtkWidget *toggle = fl_gtk_check_button_new_with_label(Fl_File_Chooser::hidden_label);
-  fl_gtk_widget_show(toggle);
   fl_gtk_file_chooser_set_extra_widget((GtkFileChooser *)gtkw_ptr, toggle);
   fl_g_signal_connect_data(toggle, "toggled", G_CALLBACK(hidden_files_cb), gtkw_ptr, NULL, (GConnectFlags) 0);
-  fl_gtk_widget_show(gtkw_ptr);
+  Fl_Window* firstw = Fl::first_window();
+  fl_gtk_widget_show_now(gtkw_ptr); // map the GTK window on screen
+  if (firstw) {
+    GdkWindow* gdkw = fl_gtk_widget_get_window(gtkw_ptr);
+    Window xw = fl_gdk_x11_drawable_get_xid(gdkw); // get the X11 ref of the GTK window
+    XSetTransientForHint(fl_display, xw, fl_xid(firstw)); // set the GTK window transient for the last FLTK win
+    }
   gboolean state = fl_gtk_file_chooser_get_show_hidden((GtkFileChooser *)gtkw_ptr);
   fl_gtk_toggle_button_set_active((GtkToggleButton *)toggle, state);
   
-  if (fl_gtk_dialog_run ((GtkDialog *)gtkw_ptr) == GTK_RESPONSE_ACCEPT) {
+  gint response_id = GTK_RESPONSE_NONE;
+  fl_g_signal_connect_data(gtkw_ptr, "response", G_CALLBACK(run_response_handler), &response_id, NULL, (GConnectFlags) 0);
+  while (response_id == GTK_RESPONSE_NONE) { // loop that shows the GTK dialog window
+    fl_gtk_main_iteration(); // one iteration of the GTK event loop
+    while (XEventsQueued(fl_display, QueuedAfterReading)) { // emulate modal dialog
+      XEvent xevent;
+      XNextEvent(fl_display, &xevent);
+      if (!fl_find(xevent.xany.window)) continue; // skip events to non-FLTK windows
+      if ( xevent.type == Expose ) fl_handle(xevent); // ignore non Expose events
+    }
+    Fl::flush(); // do the drawings needed after Expose events
+  } 
+  
+  if (response_id == GTK_RESPONSE_ACCEPT) {
     if (_parsedfilt) {
       GtkFileFilter *gfilter = fl_gtk_file_chooser_get_filter((GtkFileChooser *)gtkw_ptr);
       for (_filtvalue = 0; _filtvalue < _nfilters; _filtvalue++) {
@@ -539,7 +563,8 @@ int Fl_GTK_File_Chooser::fl_gtk_chooser_wrapper()
     }
   }
   delete[] filter_tab;
-  fl_gtk_widget_hide (gtkw_ptr);
+  if ( response_id == GTK_RESPONSE_DELETE_EVENT) gtkw_ptr = NULL;
+  else fl_gtk_widget_hide (gtkw_ptr);
   
   // I think this is analogus to doing an Fl::check() - we need this here to make sure
   // the GtkFileChooserDialog is removed from the display correctly
@@ -569,7 +594,7 @@ static void* fl_dlopen(const char *filename1, const char *filename2)
  * will allow us to create a GtkFileChooserDialog() on the fly,
  * without linking to the GTK libs at compile time.
  */
-void Fl_GTK_File_Chooser::probe_for_GTK_libs(void) {
+void mgFl_GTK_File_Chooser::probe_for_GTK_libs(void) {
   void *ptr_glib    = NULL;
   void *ptr_gtk     = NULL;
   
@@ -622,7 +647,6 @@ void Fl_GTK_File_Chooser::probe_for_GTK_libs(void) {
   GET_SYM(gtk_file_chooser_set_create_folders, ptr_gtk);
   GET_SYM(gtk_file_chooser_get_select_multiple, ptr_gtk);
   GET_SYM(gtk_widget_hide, ptr_gtk);
-  GET_SYM(gtk_dialog_run, ptr_gtk);
   GET_SYM(gtk_file_chooser_get_filename, ptr_gtk);
   GET_SYM(gtk_file_chooser_get_filenames, ptr_gtk);
   GET_SYM(gtk_main_iteration, ptr_gtk);
@@ -637,7 +661,9 @@ void Fl_GTK_File_Chooser::probe_for_GTK_libs(void) {
   GET_SYM(gtk_file_filter_set_name, ptr_gtk);
   GET_SYM(gtk_file_filter_get_name, ptr_gtk);
   GET_SYM(gtk_file_chooser_set_extra_widget, ptr_gtk);
-  GET_SYM(gtk_widget_show, ptr_gtk);
+  GET_SYM(gtk_widget_show_now, ptr_gtk);
+  GET_SYM(gtk_widget_get_window, ptr_gtk);
+  GET_SYM(gdk_x11_drawable_get_xid, ptr_gtk);
   GET_SYM(gtk_check_button_new_with_label, ptr_gtk);
   GET_SYM(g_signal_connect_data, ptr_gtk);
   GET_SYM(gtk_toggle_button_get_active, ptr_gtk);
@@ -647,7 +673,3 @@ void Fl_GTK_File_Chooser::probe_for_GTK_libs(void) {
   
   did_find_GTK_libs = 1;
 } // probe_for_GTK_libs
-
-//
-// End of "$Id: $".
-//
