@@ -661,13 +661,13 @@ return command;
 }
 
 
-extern void fix_paste_timeout(void *u);
+//extern void fix_paste_timeout(void *u);
 
 void align_menu_callback(Fl_Widget *ob, void *data)
 {
   SEA_VIEW *view = (SEA_VIEW *) ob->user_data();
   int reponse = ((Fl_Menu_*)ob)->mvalue() - view->menu_align;
-	Fl::remove_timeout(fix_paste_timeout);
+	//Fl::remove_timeout(fix_paste_timeout);
 	if(reponse <= ALIGN_SITES) { 
 	  align_selected_parts(view, (align_choice)reponse );
 	  if(is_view_valid(view)) {
@@ -691,7 +691,7 @@ void align_menu_callback(Fl_Widget *ob, void *data)
 		view->DNA_obj->redraw();
 		view->horsli->redraw();
 	}
-	Fl::add_timeout(0.5, fix_paste_timeout);
+	//Fl::add_timeout(0.5, fix_paste_timeout);
 }
 
 
