@@ -36,10 +36,10 @@ void gets_no_echo(char *password, size_t lpw);
 int trim_key(char *name) /* remove trailing spaces */
 {
 char *p;
-int l = strlen(name);
+int l = (int)strlen(name);
 p = name + l - 1;
 while( p >= name && *p == ' ' ) *(p--) = 0;
-return (p + 1) - name;
+return (int)((p + 1) - name);
 }
 
 
@@ -48,7 +48,7 @@ void padtosize(char *pname, char *name, int length)
 int i;
 strncpy(pname,name,length);
 pname[length]=0;
-for(i=strlen(pname);i<length;i++) pname[i]=' ';
+for(i=(int)strlen(pname);i<length;i++) pname[i]=' ';
 }
 
 
@@ -353,7 +353,7 @@ for(i = 1; i <= 3; i++) {
 		break;
 		}
 	else
-		base = (p-nucleotides)/2;
+		base = (int)(p-nucleotides)/2;
 	num = num * 4 + nucnum[base];
 	codon++;
 	}
@@ -405,10 +405,10 @@ char *fin;
 int l, flag=1;
 
 if(l1 > 0) {
-	if( (fin = (char *)memchr(s1, 0, l1) ) != NULL) l1 = fin - s1;
+	if( (fin = (char *)memchr(s1, 0, l1) ) != NULL) l1 = (int)(fin - s1);
 	}
 if(l2 > 0) {
-	if( (fin = (char *)memchr(s2, 0, l2) ) != NULL) l2 = fin - s2;
+	if( (fin = (char *)memchr(s2, 0, l2) ) != NULL) l2 = (int)(fin - s2);
 	}
 
 if(l2 > l1) {
@@ -443,12 +443,12 @@ void compact(char *chaine)
 int l;
 char *p, *q;
 
-l=strlen(chaine); p=chaine+l;
+l= (int)strlen(chaine); p=chaine+l;
 while( *(--p) == ' ' && p>=chaine) *p=0;
 while((p=strchr(chaine,' '))!=NULL) {
 	q=p+1;
 	while(*q==' ') q++;
-	l=q-p;
+	l= (int)(q-p);
 	while(*q!=0) {*(q-l) = *q; q++; }
 	*(q-l)=0;
 	}
@@ -599,7 +599,7 @@ static char vcible[151];
 
 pos=cible+lcible-1;
 while( pos>=cible && *pos==' ' ) pos--;
-lcible=pos-cible+1;
+lcible= (int)(pos-cible+1);
 memcpy(vcible,cible,lcible);
 vcible[lcible]=0;
 cible=vcible;
@@ -609,7 +609,7 @@ else
 	total=nbrmots-2;
 
 if(posmot[0]!=NULL) { /* comparaison avec mot initial */
-	l=strlen(posmot[0]);
+	l= (int)strlen(posmot[0]);
 	if(strncmp(cible,posmot[0],l)!=0) return 0;
 	cible += l;
 	num++;
@@ -618,13 +618,13 @@ while(num<total) { /* recherche des mots internes */
 	num++;
 	pos=strstr(cible,posmot[num]);
 	if(pos==NULL) return 0;
-	l=strlen(posmot[num]);
+	l= (int)strlen(posmot[num]);
 	cible = pos+l;
 	num++;
 	}
 if( total==nbrmots-1 ) return 1; /* template se termine par @ */
 /* test si cible se termine par dernier mot du template */
-l=strlen(posmot[nbrmots-1]); 
+l= (int)strlen(posmot[nbrmots-1]);
 if( strcmp(vcible+lcible-l,posmot[nbrmots-1]) == 0 ) return 1;
 return 0;
 }
