@@ -14,9 +14,12 @@
 #include <sys/time.h>
 #include <termios.h>
 #elif defined(WIN32)
+#if _WIN32_WINNT < 0x0501
+#define _WIN32_WINNT  0x0501
+#endif
 #include <Winsock2.h>
-#include <Ws2tcpip.h>
-#include <Wspiapi.h>
+#include <Ws2tcpip.h>  // for getaddrinfo, freeaddrinfo, struct addrinfo
+//#include <Wspiapi.h>
 #endif
 
 
@@ -389,7 +392,7 @@ return values :
 {
 Reponse *rep;
 char *reponse, *code, *p, *challenge;
-int codret, totspecs, totkeys;
+int codret, totspecs, totkeys, i;
 
 if(raa_current_db == NULL) return nosocket;
  sock_printf(raa_current_db, "acnucopen&db=%s\n", db_name);
@@ -466,6 +469,7 @@ raa_current_db->longa=(raa_current_db->maxa-1)/(8 * sizeof(int))+1;
 	raa_current_db->WIDTH_KW = 40;
 	raa_current_db->lrtxt = 60;
 	raa_current_db->SUBINLNG=63;
+        raa_current_db->VALINSHRT2=0;
 	raa_current_db->ACC_LENGTH=13; /* conservative value */
 p = val(rep,"L_MNEMO");
 if(p != NULL) { raa_current_db->L_MNEMO = atoi(p); free(p); }
@@ -489,7 +493,20 @@ if(p != NULL) {
 	free(p); 
 	raa_current_db->rlng_buffer = (struct rlng *)calloc((raa_current_db->SUBINLNG + 1), sizeof(int));
 	}
-
+p = val(rep,"VALINSHRT2");
+if (p != NULL) {
+  raa_current_db->VALINSHRT2 = atoi(p); free(p);
+}
+  int valinshrt = (raa_current_db->VALINSHRT2 ? raa_current_db->VALINSHRT2 : 1);
+    for (i = raa_sub_of_bib; i <= raa_acc_of_loc; i++) {
+      int size = 100 * valinshrt;
+      raa_current_db->readshrt2_data[i] = (struct shrt2_list*)malloc(sizeof(int) * (size + 4));
+      raa_current_db->readshrt2_data[i]->size = size;
+      raa_current_db->readshrt2_data[i]->length = 0;
+      raa_current_db->readshrt2_data[i]->point = 0;
+      raa_current_db->readshrt2_data[i]->next = 0;
+    }
+  
 clear_reponse(rep);
 return 0;
 }
@@ -1908,6 +1925,54 @@ return raa_readshrt(raa_current_db, point, pval);
 }
 
 
+static void load_shrt2_aux(raa_db_access *raa_current_db, unsigned point, raa_shortl2_kind kind)
+{
+  static const char kind_name[][11] = {"sub_of_bib", "spc_of_loc", "bib_of_loc", "aut_of_bib", "bib_of_aut",
+    "sub_of_acc", "key_of_sub", "acc_of_loc"};
+  char *p, *reponse;
+  int i;
+  
+  if (raa_current_db == NULL) return;
+  sock_printf(raa_current_db, "followshrt2&num=%u&kind=%s&rank=0&max=%d\n", point, kind_name[kind], 
+              raa_current_db->readshrt2_data[kind]->size);
+  /* reponse is:  code=0&num=xx&rank=xx&n=xx&val,.... n times ...\n  */
+  reponse = read_sock(raa_current_db);
+  
+  if (reponse == NULL || strncmp(reponse, "code=0&", 7) != 0) {
+    return;
+  }
+  raa_current_db->readshrt2_data[kind]->point = point;
+  p = strstr(reponse, "num=");
+  raa_current_db->readshrt2_data[kind]->next = atoi(p + 4);
+  p = strstr(reponse, "n=");
+  raa_current_db->readshrt2_data[kind]->length = atoi(p + 2);
+  p = strchr(p, '&') + 1;
+  for (i = 0; i < raa_current_db->readshrt2_data[kind]->length; i++) {
+    sscanf(p, "%u", raa_current_db->readshrt2_data[kind]->vals+i);
+    p = strchr(p, ',') + 1;
+  }
+}
+
+
+unsigned raa_followshrt2(raa_db_access *raa_current_db, int *p_point, int *p_rank, raa_shortl2_kind kind)
+{
+  int rank = *p_rank;
+  unsigned retval;
+  
+  if (raa_current_db->readshrt2_data[kind]->point == *p_point &&
+      rank < raa_current_db->readshrt2_data[kind]->length) {
+    retval = raa_current_db->readshrt2_data[kind]->vals[rank++];
+    if (rank >= raa_current_db->readshrt2_data[kind]->length) {
+      *p_rank = 0;
+      *p_point = raa_current_db->readshrt2_data[kind]->next;
+    }
+    else *p_rank = rank;
+    return retval;
+  }
+  load_shrt2_aux(raa_current_db, *p_point, kind);
+  return raa_followshrt2(raa_current_db, p_point, p_rank, kind);
+}
+
 
 char *raa_ghelp(raa_db_access *raa_current_db, char *fname, char *topic)
 /* returns all help topic in one string in private memory
@@ -2611,15 +2676,15 @@ return raa_current_db->translate_buffer;
 char raa_translate_init_codon(raa_db_access *raa_current_db, int numseq)
 {
 char codon[4];
-int point, special_init = TRUE, val, gc, phase;
+int point, special_init = TRUE, val, gc, phase, rank = 0;
 
 if (raa_current_db->num_5_partial == 0) raa_current_db->num_5_partial = raa_iknum(raa_current_db, "5'-PARTIAL", raa_key);
 raa_readsub(raa_current_db, numseq, NULL, NULL,NULL, &point, NULL, &phase, &gc);
 if(phase != 0) special_init = FALSE;
 else	{ /* la seq est-elle 5'-PARTIAL ? */
-	while(point != 0) {
-		point = raa_readshrt(raa_current_db, point, &val);
-		if(val == raa_current_db->num_5_partial) {
+	while (point != 0) {
+		val = raa_followshrt2(raa_current_db, &point, &rank, raa_key_of_sub);
+		if (val == raa_current_db->num_5_partial) {
 			special_init = FALSE;
 			break;
 			}
