@@ -7,7 +7,6 @@
 #include <ctype.h>
 #if defined(WIN32)
 #define SOCKBUFS 8192
-#include <winsock2.h>
 #endif
 #ifdef __alpha
 typedef long raa_long;
@@ -71,6 +70,14 @@ struct readshrt_aux {
 	 int shrt_max, shrt_begin, total;
 	};
 
+typedef struct shrt2_list {
+  int length;
+  int size;
+  unsigned point;
+  unsigned next;
+  unsigned vals[1];
+} shrt2_list;
+
 struct readsmj_aux {
 	 int lastrec;
 	 char **names, **libels;
@@ -95,12 +102,15 @@ struct raa_pair {
 
 #define WIDTH_MAX 150
 
+typedef enum { raa_sub_of_bib = 0, raa_spec_of_loc, raa_bib_of_loc, raa_aut_of_bib, raa_bib_of_aut,
+  raa_sub_of_acc, raa_key_of_sub, raa_acc_of_loc } raa_shortl2_kind;
+
 typedef struct _raa_db_access {
 	char *dbname;
 	FILE *raa_sockfdr, *raa_sockfdw;
 	int genbank, embl, swissprot, nbrf;
 	int nseq, longa, maxa;
-	int L_MNEMO, WIDTH_SP, WIDTH_KW, WIDTH_SMJ, WIDTH_AUT, WIDTH_BIB, ACC_LENGTH, SUBINLNG, lrtxt;
+	int L_MNEMO, WIDTH_SP, WIDTH_KW, WIDTH_SMJ, WIDTH_AUT, WIDTH_BIB, ACC_LENGTH, SUBINLNG, lrtxt, VALINSHRT2;
 	raa_node **sp_tree; /* NULL or the full taxonomy tree */
 	int max_tid; /* largest correct taxon ID value */
 	int *tid_to_rank; /* NULL or tid-to-rank table */
@@ -112,6 +122,7 @@ typedef struct _raa_db_access {
 	struct readsp_kw_aux readspec_data, readkey_data;
 	struct nextelt_aux nextelt_data;
 	struct readshrt_aux readshrt_data;
+	struct shrt2_list *readshrt2_data[raa_acc_of_loc+1];
 	struct readsmj_aux readsmj_data;
 	void *matchkey_data;
 	/* mostly for raa_query/raa_query_win usage */
@@ -122,7 +133,7 @@ typedef struct _raa_db_access {
 	char sock_input[SOCKBUFS]; /* WIN32 socket input buffer */
 	char *sock_input_pos, *sock_input_end;
 	char sock_output[SOCKBUFS]; /* WIN32 socket output buffer */
-	DWORD sock_output_lbuf;
+	unsigned sock_output_lbuf;
 #endif
   char buffer[5000];
   char remote_file[300];
@@ -207,6 +218,7 @@ char *raa_readacc(raa_db_access  *raa_current_db, int num, int *plsub);
 int raa_readext(raa_db_access  *raa_current_db, int num, int *mere, int *deb, int *fin);
 int raa_readlng(raa_db_access  *raa_current_db, int num);
 unsigned raa_readshrt(raa_db_access  *raa_current_db, unsigned point, int *val);
+unsigned raa_followshrt2(raa_db_access *raa_current_db, int *p_point, int *p_rank, raa_shortl2_kind kind);
 char *raa_ghelp(raa_db_access  *raa_current_db, char *fname, char *topic);
 int raa_nextmatchkey(raa_db_access  *raa_current_db, int num, char *pattern, char **matching);
 int raa_savelist(raa_db_access  *raa_current_db, int lrank, FILE *out, int use_acc, char *prefix);
