@@ -4,7 +4,6 @@
 #elif defined(__APPLE__) 
 #include <unistd.h>
 #include <util.h>
-#include <CoreServices/CoreServices.h>
 
 #else //X11
 #include <unistd.h>
@@ -18,9 +17,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <ctype.h>
 #include <FL/x.H>
 #include <FL/Fl.H>
-#include <FL/Fl_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Return_Button.H>
 #include <FL/Fl_Multiline_Output.H>
@@ -38,39 +38,43 @@ struct listener_data {
 #else
 	pid_t pid;
 #endif
+	int fd;
 	Fl_Button *ok, *interrupt;
 	Fl_Scrollbar *bar;
-	int nl;
-	int view_nl;
-	int modified;
-	char *from, *next, *eol;
-	char buffer[10000000];
+	int nl; // current number of complete lines in buffer
+	int view_nl; // max no of visible lines in Fl_Multiline_Output
+	char *from; // start in buffer of displayed text during running
+	char *next; // just after last displayed buffer byte
+	char *eol; // points to where new bytes will be put in buffer
+	bool modified;
+	char buffer[10000000]; // buffer to hold text. Start from beginning again when more text than buffer size
 };
 
-void ext_prog_display(void *ptr);
+typedef void (*thread_function_t) (void *);
+extern int fl_create_thread(thread_function_t f, void* p, unsigned stack_size_KB);
 
-static void inter_callback(Fl_Widget *o, void *p)
+
+static void inter_callback(Fl_Widget *o, struct listener_data *data)
 {
-	struct listener_data *data = (struct listener_data *)p;
 	data->status = aborted;
 #ifdef WIN32
-	TerminateProcess(data->pid, 0);
-	Sleep(500 /*msec*/); //some delay seems necessary between TerminateProcess and deletion of open files
+	BOOL ok = TerminateProcess(data->pid, 0);
+	if (ok) {
+	  while ( WaitForSingleObject(data->pid, 0) != WAIT_OBJECT_0) Fl::wait(0.1);
+	  }
 #else
 	kill(data->pid, SIGKILL);
 #endif
 }
 
-static void ok_callback(Fl_Widget *o, void *p)
+static void ok_callback(Fl_Widget *o, struct listener_data *data)
 {
-	struct listener_data *data = (struct listener_data *)p;
 	data->status = closed;
 }
 
-static void bar_callback(Fl_Widget *wgt, void *data)
+static void bar_callback(Fl_Scrollbar *wgt, Fl_Multiline_Output *o)
 {
-	Fl_Multiline_Output *o = (Fl_Multiline_Output *)data;
-	int first_line = ((Fl_Scrollbar *)wgt)->value();
+	int first_line = wgt->value();
 	const char *p = o->value();
 	do {
 		if(*p == '\n') first_line--;
@@ -82,169 +86,127 @@ static void bar_callback(Fl_Widget *wgt, void *data)
 }
 
 
-static void copy_callback(Fl_Widget *wgt, void *p)
+static void copy_callback(Fl_Widget *wgt, Fl_Multiline_Output *o)
 {
-	Fl_Multiline_Output *o = (Fl_Multiline_Output *)p;
 	o->position(o->size(), 0);
 	o->copy(1);
 }
 
 
-void ext_prog_display(void *ptr)
-//display data from memory to FLTK viewer
-{
-	struct listener_data *data = (struct listener_data *)ptr;
-	if(data->modified) {
-		Fl_Multiline_Output *o = data->o;
-		o->static_value(data->from, data->next - data->from);
-		o->redraw();
-		data->modified = 0;
-		}
-	Fl::repeat_timeout(0.5, ext_prog_display, ptr);
+void ext_prog_listener(struct listener_data *data)
+{//transmits data from fd to FLTK viewer with CR emulation
+  char *p;
+  static char fd_buff[2000];
+  int l;
+  while (data->status == running) {
+    l = read(data->fd, fd_buff, sizeof(fd_buff));
+    if (l <= 0) break;
+    Fl::lock();
+    if (data->next + l >= data->buffer + sizeof(data->buffer)) {
+      data->next = data->from = data->eol = data->buffer;
+      data->nl = 0;
+    }
+    for (p = fd_buff; p < fd_buff + l; p++) {
+      if (*p == '\r') {
+	if (p+1 >= fd_buff + l || *(p+1) != '\n') {
+	  while(data->eol > data->buffer && *(data->eol - 1) != '\n') 
+			      (data->eol)--;
+	  }
 	}
-
-void ext_prog_listener(int fd, void *ptr)
-{//transmits data from fd to memory with CR emulation
-	char *p;
-	struct listener_data *data = (struct listener_data *)ptr;
-	static char fd_buff[10000];
-	Fl_Multiline_Output *o = data->o;
-	int l;
-	if(data->next + sizeof(fd_buff) >= data->buffer + sizeof(data->buffer)) {
-		data->next = data->from = data->eol = data->buffer;
-		data->nl = 0;
-		}
-	l = read(fd, fd_buff, sizeof(fd_buff));
-	if(l > 0) {
-		data->modified = 1;
-		for(p = fd_buff; p < fd_buff + l; p++) {
-			if(*p == '\r') {
-				if(p+1 >= fd_buff + l || *(p+1) != '\n') {
-					while(data->eol > data->buffer && *(data->eol - 1) != '\n') 
-						(data->eol)--;
-					}
-				}
-			else {
-				if(*p == '\n') {
-					data->eol = data->next;
-					data->nl++;
-					if(data->nl >= data->view_nl) data->from = strchr(data->from, '\n') + 1;
-					}
-				*(data->eol++) = *p;
-				if(data->eol > data->next) data->next = data->eol;
-				}
-			}
-		}
-	else {
-		Fl::remove_timeout(ext_prog_display, ptr);
-		o->static_value(data->buffer, data->next - data->buffer);
-		o->position(data->next - data->buffer, data->next - data->buffer);
-		data->bar->value(data->nl, data->view_nl, 0, data->nl);
-		data->bar->show();
-		if(!data->ok->active()) {
-			data->ok->activate();
-			data->interrupt->label("Cancel");
-			data->status = completed;
-			}
-		}
+      else {
+	if (*p == '\n') {
+	  data->eol = data->next;
+	  data->nl++;
+	  if(data->nl >= data->view_nl) data->from = strchr(data->from, '\n') + 1;
+	  }
+	*(data->eol++) = *p;
+	if(data->eol > data->next) data->next = data->eol;
+	}
+      }
+    data->modified = true;
+    Fl::unlock();
+    }
+  Fl::lock();
+  if (data->status == running) {
+    data->o->static_value(data->buffer, data->next - data->buffer);
+    data->o->position(data->next - data->buffer, data->next - data->buffer);
+    data->o->redraw();
+    data->bar->value(data->nl, data->view_nl, 0, data->nl);
+    data->bar->show();
+    if (!data->ok->active()) {
+      data->ok->activate();
+      data->interrupt->label("Cancel");
+      data->status = completed;
+      }
+    }
+  Fl::unlock();
 }
 
 
-Fl_Window *create_and_run_pseudoterminal(const char *label, struct listener_data *data,
-#ifdef WIN32
-										 HANDLE pipe, HANDLE pid
-#else
-										 int masterfd, pid_t pid
-#endif
-								   )
+Fl_Window *create_and_run_pseudoterminal(const char *label, struct listener_data *data)
 #define FL_min(a,b)      ( (a) < (b) ? (a):(b) )
 {
-	static char message[100];
-	Fl_Window *w = new Fl_Window( FL_min(700, Fl::w()), FL_min(600, Fl::h()-22 ) );
-	w->xclass("Terminal");
-	w->label(label);
-	w->callback(inter_callback, data);
-	Fl_Multiline_Output *o = new Fl_Multiline_Output(0, 3, w->w() - 15, w->h() - 40);
-	o->textfont(FL_COURIER);
-	o->textsize(12);
-	o->maximum_size(5000000);
-	data->bar = new Fl_Scrollbar(o->x() + o->w(), o->y(), 15, o->h());
-	data->bar->callback(bar_callback, o);
-	data->view_nl = o->h() / o->textsize();
+  static char message[100];
+  Fl_Double_Window *w = new Fl_Double_Window( FL_min(700, Fl::w()), FL_min(600, Fl::h()-22 ) );
+  w->xclass("Terminal");
+  w->label(label);
+  w->callback((Fl_Callback*)inter_callback, data);
+  Fl_Multiline_Output *o = new Fl_Multiline_Output(0, 3, w->w() - 15, w->h() - 40);
+  o->textfont(FL_COURIER);
+  o->textsize(12);
+  o->maximum_size(5000000);
+  data->bar = new Fl_Scrollbar(o->x() + o->w(), o->y(), 15, o->h());
+  data->bar->callback((Fl_Callback*)bar_callback, o);
+  data->view_nl = o->h() / o->textsize();
 #ifdef WIN32
-	data->view_nl = (int)(data->view_nl * 0.80);
+  data->view_nl = (int)(data->view_nl * 0.80);
 #endif
-	data->bar->value(0, data->view_nl, 0, data->view_nl);
-	data->bar->hide();
-	sprintf(message, "Wait for %s completion", label);
-	Fl_Box *w_message = new Fl_Box(0, o->y() + o->h() + 3, w->w(), 20, message);
-	Fl_Button *w_copy = new Fl_Button(w->w()/2 - 30, w_message->y(), 60, 20, "Copy all");
-	w_copy->hide();
-	w_copy->callback(copy_callback, o);
-	data->status = running;
-	data->o = o;
-	data->pid = pid;
-	data->nl = 0;
-	data->from = data->next = data->eol = data->buffer;
-	data->modified = 0;
-	Fl_Button *interrupt = new Fl_Button(5, w->h() - 35, 70, 30, "Interrupt");
-	interrupt->callback(inter_callback, data);
-	interrupt->labelfont(FL_HELVETICA_BOLD);
-	Fl_Return_Button *ok = new Fl_Return_Button(w->w() - 60, interrupt->y(), 50, 30, "OK");
-	ok->callback(ok_callback, data);
-	ok->labelfont(FL_HELVETICA_BOLD);
-	ok->deactivate();
-	data->ok = ok;
-	data->interrupt = interrupt;
-	w->end();
-    w->position( (Fl::w() - w->w())/2, (Fl::h() - w->h())/2 );
-	Fl_Box *r = new Fl_Box(interrupt->x() + interrupt->w(), o->y(), ok->x() - interrupt->x() - interrupt->w(), o->h());
-	w->resizable(r);
-	w->show();
+  data->bar->value(0, data->view_nl, 0, data->view_nl);
+  data->bar->hide();
+  sprintf(message, "Wait for %s completion", label);
+  Fl_Box *w_message = new Fl_Box(0, o->y() + o->h() + 3, w->w(), 20, message);
+  Fl_Button *w_copy = new Fl_Button(w->w()/2 - 30, w_message->y(), 60, 20, "Copy all");
+  w_copy->hide();
+  w_copy->callback((Fl_Callback*)copy_callback, o);
+  data->status = running;
+  data->o = o;
+  data->nl = 0;
+  data->from = data->next = data->eol = data->buffer;
+  Fl_Button *interrupt = new Fl_Button(5, w->h() - 35, 70, 30, "Interrupt");
+  interrupt->callback((Fl_Callback*)inter_callback, data);
+  interrupt->labelfont(FL_HELVETICA_BOLD);
+  Fl_Return_Button *ok = new Fl_Return_Button(w->w() - 60, interrupt->y(), 50, 30, "OK");
+  ok->callback((Fl_Callback*)ok_callback, data);
+  ok->labelfont(FL_HELVETICA_BOLD);
+  ok->deactivate();
+  data->ok = ok;
+  data->interrupt = interrupt;
+  data->modified = false;
+  w->end();
+  w->position( (Fl::w() - w->w())/2, (Fl::h() - w->h())/2 );
+  Fl_Box *r = new Fl_Box(interrupt->x() + interrupt->w(), o->y(), ok->x() - interrupt->x() - interrupt->w(), o->h());
+  w->resizable(r);
+  w->show();
 #ifndef MICRO
-	w->hotspot(w);
+  w->hotspot(w);
 #endif
+  fl_create_thread( (thread_function_t)ext_prog_listener, data, 0);
+  while (data->status == running) {
+    if (data->modified) {
+      data->modified = false;
+      data->o->static_value(data->from, data->next - data->from);
+      data->o->redraw();
+      }
+    Fl::wait(0.1);
+    }
+  w_message->hide();
+  w_copy->show();
 #ifdef WIN32
-	//add_fd does not work well under WIN32: Fl::wait() blocks in read(fd when nothing comes in the pipe
-	//so an equivalent is done outside from Fl::wait()
-	int masterfd = _open_osfhandle(
-#ifdef _WIN64
-				       (intptr_t)
+  _close(data->fd);
 #else
-				       (long)
+  close(data->fd);
 #endif
-				       pipe, _O_RDONLY);
-#else
-	Fl::add_fd(masterfd, FL_READ | FL_EXCEPT, ext_prog_listener, data);
-#endif
-	Fl::add_timeout(0.5, ext_prog_display, data);
-	while(data->status == running) {
-#ifdef WIN32
-		while(TRUE) {
-			DWORD avail = 0;
-			PeekNamedPipe(pipe, NULL,0,NULL, &avail, NULL);//is there data in pipe?
-			if(avail <= 0) break;
-			ext_prog_listener(masterfd, data);//read data from pipe
-			Fl::wait(0);//process any interface event, especially interrupt requests & timeout
-			}
-		if( WaitForSingleObject(pid, 0) == WAIT_OBJECT_0) {//is pid dead?
-			//complete reading from pipe and detect closed pipe
-			while(data->status == running) ext_prog_listener(masterfd, data);
-			}
-		Fl::wait(0.1);//process any interface event, especially interrupt requests & timeout
-#else
-		Fl::wait();
-#endif
-		}
-	Fl::remove_timeout(ext_prog_display, data);
-	w_message->hide();
-	w_copy->show();
-#ifdef WIN32
-	_close(masterfd);
-#else
-	Fl::remove_fd(masterfd);
-#endif
-	return w;
+  return w;
 }
 
 
@@ -253,7 +215,7 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
  cmd can be:
  prog arg1 arg2 ...
  or:
- prog arg1 arg2 ... > outfile
+ "prog" arg1 "arg2" ... > outfile
  and:
  prog can also use file named in dialogfname as stdin (NULL if no such file)
  label: short name of operation that is run
@@ -274,7 +236,7 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 	SECURITY_ATTRIBUTES saAttr; 
 	PROCESS_INFORMATION pi; 
 	STARTUPINFOW si;
-	char *p;
+	char *p, *q;
 	
 	if(dialogfname != NULL) {
 		FILE *in = fopen(dialogfname, "r");
@@ -291,6 +253,10 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 		*p = 0;
 		p++;
 		while(*p == ' ') p++;
+		if (*p == '"') {
+		  q = strchr(++p, '"');
+		  if (q) *q = 0;
+		  }
 		char *stdoutfname = strdup(p);
 		FILE *out = fopen(stdoutfname, "w");
 		free(stdoutfname);
@@ -328,7 +294,15 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 	  return 1;
 	}
 	CloseHandle(pi.hThread);
-	w = create_and_run_pseudoterminal(label, &data, ChildStdoutRd, pi.hProcess);
+	data.pid = pi.hProcess;
+	data.fd = _open_osfhandle(
+#  ifdef _WIN64
+				 (intptr_t)
+#  else
+				 (long)
+#  endif
+				 ChildStdoutRd, _O_RDONLY);
+	w = create_and_run_pseudoterminal(label, &data);
 	CloseHandle(pi.hProcess);
 #else
   // creates a pseudoterminal
@@ -359,7 +333,9 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 	  return 1;
 	  }
 	else if(pid > 0) {// the parent
-		w = create_and_run_pseudoterminal(label, &data, masterfd, pid);
+		data.pid = pid;
+		data.fd = masterfd;
+		w = create_and_run_pseudoterminal(label, &data);
 	}
 	else {//the child
 		if(dialogfname != NULL) {//case of use of dialog file
@@ -392,13 +368,14 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 	  // bug in Mac OS X 10.4 i386 requires us to close stdout and reopen it on fd = 1
 		fclose(stdout);
 		if((q = strchr(p, '>')) != NULL) {//open desired stdout file
-			char stdoutfile[150];
+			char stdoutfile[200];
 			FILE *f;
 			*q = 0;
 			do q++; while(*q == ' ');
+			if (*q == '"') q++;
 			strcpy(stdoutfile, q);
 			q = stdoutfile + strlen(stdoutfile) - 1;
-			while(q > stdoutfile && *q == ' ') q--;
+			while (q > stdoutfile && (*q == ' ' || *q == '"') ) q--;
 			*(q + 1) = 0;
 			f = fopen(stdoutfile, "w");
 			if(f) newout = fileno(f);
@@ -412,22 +389,41 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 		//count args
 		argc = 1;
 		q = p - 1;
-		while (*q != 0) {
-			q++;
-			if(*q == ' ') {
-				argc++;
-				do q++; while (*q == ' ');
-				}
-			}
-		if(*(q-1)==' ')argc--;
+		while (*++q) {
+		  if (isspace(*q)) continue;
+		  argc++;
+		  if (*q == '"') {
+		    q = strchr(q + 1, '"');
+		    if (!q) break;
+		    }
+		  else {
+		    do q++; while ( *q && !isspace(*q) );
+		    q--;
+		    }
+		  }
 		//put args in argv array
 		argv = (char **)malloc((argc + 1)*sizeof(char *));
 		argv[0] = progpath;
 		argv[argc] = NULL;
-		if(argc > 1) {
-			argv[1] = strtok(p, " ");
-			for(int i = 2; i < argc; i++) argv[i] = strtok(NULL, " ");
-			}
+		q = p - 1;
+		int i = 1;
+		while (*++q) {
+		  if (isspace(*q)) continue;
+		  if (*q == '"') {
+		    argv[i++] = ++q;
+		    q = strchr(q, '"');
+		    if (!q) break;
+		    *q = 0;
+		  }
+		  else {
+		    argv[i++] = q;
+		    do q++; while ( *q && !isspace(*q) );
+		    if (*q == 0) {
+		      break;
+		      }
+		    *q = 0;
+		  }
+		}
 		int err = 1; const char *errmess;
 		if(access(progpath, X_OK) != 0) errmess = "executable";
 		else {
@@ -437,7 +433,6 @@ int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const ch
 		fprintf(stderr, "\n\n\n%s is not %s\n", progpath, errmess);
 		exit(err);
 	}
-	close(masterfd);
 #endif
 	while(data.status == completed) Fl::wait();
 	delete w;
