@@ -195,32 +195,9 @@ void create_variable_mask(SEA_VIEW *view, region *maregion)
     }
 }
 
-
-int create_gblocks_mask(SEA_VIEW *view, region *maregion)
-/* return 0 iff OK */
+int run_gblocks_dialog(int *pb5_val, int *pb4_val, int *pb3_val, int *pb2_val)
 {
-  char cmd[500];
-  char base_fname[150], fname[150], *p, *q, *last;
-  int status, segment_count = -1, num, l;
-  FILE *in;
-  list_segments *segment, *previous;
   Fl_Widget *o;
-  
-  q = get_full_path(
-		    "Gblocks"
-#ifdef WIN32
-		    ".exe"
-#endif
-		    );
-  if (!q) {
-    fl_alert("Program Gblocks not found."
-#if !(defined(WIN32) || defined(__APPLE__))
-    "\nTo use Gblocks, download it from http://molevol.cmima.csic.es/castresana/Gblocks.html\n"
-    "and put it in a directory of your PATH"
-#endif
-    );
-    return 1;
-  }
   Fl_Window *w = new Fl_Window(370,100);
   w->label("Gblocks options dialog");
   Fl_Box *b = new Fl_Box(5, 5, w->w(), 20, "Options for a less stringent selection:");
@@ -242,21 +219,48 @@ int create_gblocks_mask(SEA_VIEW *view, region *maregion)
   while(TRUE) {
     o = Fl::readqueue();
     if (!o) Fl::wait();
-    else if(o == cancel) {
-      free(maregion->name);
-      free(maregion);
-      break;
-      }
+    else if(o == cancel || o == w) {
+      return 1;
+    }
     else if(o == ret)
       break;
-    }
-  int b5_val = b5->value(); // -b5=h
-  int b4_val = b4->value(); // -b4=5
-  int b3_val = b3->value(); // -b3=4
-  int b2_val = b2->value(); // -b2=(50% of species)
+  }
+  *pb5_val = b5->value(); // -b5=h
+  *pb4_val = b4->value(); // -b4=5
+  *pb3_val = b3->value(); // -b3=4
+  *pb2_val = b2->value(); // -b2=(50% of species)
   Fl::delete_widget(w);
-  if (o == cancel) return 1;
+  if (o == cancel || o == w) return 1;
+  return 0;
+}
+
+int create_gblocks_mask(SEA_VIEW *view, region *maregion, int no_gui=false, int b5_val=0, int b4_val=0, int b3_val=0, int b2_val=0)
+/* return 0 iff OK */
+{
+  char cmd[500];
+  char base_fname[PATH_MAX], fname[PATH_MAX], *p, *q, *last;
+  int status, segment_count = -1, num, l;
+  FILE *in;
+  list_segments *segment, *previous;
+  void (*alert)(const char*,...) = (no_gui ? Fl::fatal : fl_alert);
+  
+  q = get_full_path(
+		    "Gblocks"
+#ifdef WIN32
+		    ".exe"
+#endif
+		    );
+  if (!q) {
+    alert("Program Gblocks not found."
+#if !(defined(WIN32) || defined(__APPLE__))
+    "\nTo use Gblocks, download it from http://molevol.cmima.csic.es/castresana/Gblocks.html\n"
+    "and put it in a directory of your PATH"
+#endif
+    );
+    return 1;
+  }
     
+  if (!no_gui && run_gblocks_dialog(&b5_val, &b4_val, &b3_val, &b2_val)) return 1;
   p = create_tmp_filename();
   if(p == NULL) return 1;
   strcpy(base_fname, p);
@@ -272,21 +276,29 @@ int create_gblocks_mask(SEA_VIEW *view, region *maregion)
   for (int i = 0; i < view->tot_seqs; i++) delete newnames[i];
   delete[] newnames;
   if (status) {
-    fl_alert("Cannot write to temporary file %s", fname);
+    alert("Cannot write to temporary file %s", fname);
     delete_tmp_filename(base_fname);
     return 1;
     }
-  sprintf(cmd, "\"%s\" %s -p=s -t=%c", q, fname, view->protein?'p':'d');
+  sprintf(cmd, "\"%s\" \"%s\" -p=s -t=%c", q, fname, view->protein?'p':'d');
   if (b5_val) strcat(cmd, " -b5=h");
   if (b4_val) strcat(cmd, " -b4=5");
   if (b3_val) strcat(cmd, " -b3=4");
-  if (b2_val) sprintf(cmd + strlen(cmd), " -b2=%d", view->tot_seqs/2);
+  if (b2_val) sprintf(cmd + strlen(cmd), " -b2=%d", view->tot_seqs/2 + 1);
 #ifdef WIN32
   //conversion of utf-8 cmd into a WCHAR string
   int wlen = MultiByteToWideChar(CP_UTF8, 0, cmd, -1, NULL, 0);
   WCHAR *wcmd = new WCHAR[wlen];
   MultiByteToWideChar(CP_UTF8, 0, cmd, -1, wcmd, wlen);  
-  _wsystem(wcmd);
+  //_wsystem(wcmd);
+  PROCESS_INFORMATION pi; 
+  STARTUPINFOW si;
+  ZeroMemory( &pi, sizeof(PROCESS_INFORMATION) );
+  ZeroMemory( &si, sizeof(STARTUPINFOW) );
+  l = CreateProcessW(NULL, wcmd, 0,0,FALSE,CREATE_NO_WINDOW,0,0, &si, &pi);
+  if (l) {
+    while ( WaitForSingleObject(pi.hProcess, 0) != WAIT_OBJECT_0) Fl::wait(0.1);
+  }
   delete[] wcmd;
 #else
   strcat(cmd, " > /dev/null");
@@ -308,7 +320,7 @@ int create_gblocks_mask(SEA_VIEW *view, region *maregion)
       }
     }
   if (status == 0 && segment_count == 0) {
-    fl_alert("Gblocks didn't find any block of conserved sites");
+    alert("Gblocks didn't find any block of conserved sites");
     status = 1;
     }
   if (!status) {
@@ -343,10 +355,6 @@ int create_gblocks_mask(SEA_VIEW *view, region *maregion)
     if (num < segment_count) status = 1;
     maregion->hide_when_viewasprots = FALSE;
     }
-  if (status) {
-    free(maregion->name);
-    free(maregion);
-    }
   if (in) fclose(in);
   delete_tmp_filename(base_fname);
   return status;
@@ -390,7 +398,11 @@ int create_codon_position_mask(SEA_VIEW *view, position_mask mask, const char *r
   
   memset(view->region_line, '-', view->seq_length);
   if (mask == gblocks_sites) {
-    if (create_gblocks_mask(view, maregion) ) return 0;
+    if (create_gblocks_mask(view, maregion) ) {
+      free(maregion->name);
+      free(maregion);
+      return 0;
+    }
   }
   else if (mask >= position_1 && mask <= position_12) {
     previous = NULL;
@@ -709,7 +721,7 @@ const char *site_mask_creation_dialog(SEA_VIEW *view, position_mask *outmask)
 	  else if(o == mask[5]) {*outmask = variable_sites;}
 	  retval = position_mask_names[*outmask];
 	}
-	else if(o == cancel) {
+	else if(o == cancel || o == w) {
 	  retval = NULL;
 	  break;
 	  }
