@@ -23,12 +23,12 @@ char *get_res_value(const char *name, const char *def_value)
   CFPropertyListRef cfvalue;
   static char value[400];
 
-  cfname = CFStringCreateWithCString(kCFAllocatorDefault, name, kCFStringEncodingASCII);
+  cfname = CFStringCreateWithCString(kCFAllocatorDefault, name, kCFStringEncodingUTF8);
   if(cfname == NULL) return (char *)def_value;
   cfvalue = CFPreferencesCopyAppValue(cfname, kCFPreferencesCurrentApplication);
   CFRelease(cfname); 
   if(cfvalue == NULL) return (char *)def_value;
-  Boolean ok = CFStringGetCString( (CFStringRef)cfvalue, value, sizeof(value), kCFStringEncodingASCII);
+  Boolean ok = CFStringGetCString( (CFStringRef)cfvalue, value, sizeof(value), kCFStringEncodingUTF8);
   CFRelease(cfvalue);
   return (ok ? value : (char *)def_value);
 }
@@ -38,9 +38,9 @@ int set_res_value(const char *name, const char *value)
 {
   CFStringRef cfname, cfvalue;
 
-  cfname = CFStringCreateWithCString(kCFAllocatorDefault, name, kCFStringEncodingASCII);
+  cfname = CFStringCreateWithCString(kCFAllocatorDefault, name, kCFStringEncodingUTF8);
   if(cfname == NULL) return 1;
-  cfvalue = CFStringCreateWithCString(kCFAllocatorDefault, value, kCFStringEncodingASCII);
+  cfvalue = CFStringCreateWithCString(kCFAllocatorDefault, value, kCFStringEncodingUTF8);
   if(cfvalue == NULL) return 1;
   CFPreferencesSetAppValue(cfname, cfvalue, kCFPreferencesCurrentApplication);
   CFPreferencesAppSynchronize(kCFPreferencesCurrentApplication);
@@ -53,7 +53,7 @@ int set_res_value(const char *name, const char *value)
 void delete_res_value(const char *name)
 {
   CFStringRef cfname;
-  cfname = CFStringCreateWithCString(kCFAllocatorDefault, name, kCFStringEncodingASCII);
+  cfname = CFStringCreateWithCString(kCFAllocatorDefault, name, kCFStringEncodingUTF8);
   if(cfname == NULL) return;
   CFPreferencesSetAppValue(cfname, NULL, kCFPreferencesCurrentApplication);
   CFRelease(cfname); 
