@@ -343,14 +343,6 @@ int freq_obs(char* seq1, char* seq2, double* freq){
 }
 
 
-static void close_win_callback(Fl_Widget *ob, void *data)
-{
-  if (!ob->as_window()) ob = ob->window();
-  ob->hide();
-  Fl::delete_widget( ob );
-}
-
-
 static void save_stat_cb(Fl_Widget *ob, void *data)
 {
   Fl_Multiline_Output *ml = (Fl_Multiline_Output*)data;
@@ -370,7 +362,7 @@ static void save_stat_cb(Fl_Widget *ob, void *data)
 void geni_act(SEA_VIEW *view)
 {
   int i, j, k, l, ii, nb, lgtot, lg, lgvar, lginfo, *lgcomp, totlgcomp, nc=0;
-  int *selected, nbaa, nbcp, h;
+  int *selected, nbaa, nbcp;
   double **comp, *compall, compcomp[4], **compgroup, *compgroupall, min[4], max[4];
   double P, Psum=0., Q, Qsum=0., Pmin, Pmax, Qmin, Qmax, titvratio, titvmin, titvmax, freq[16], mat_obs[16], *ti, *tv;
   char **genseq, **genname, *text, *minname[4], *maxname[4], nt[4]={'A', 'C', 'G', 'T'};
@@ -557,9 +549,7 @@ void geni_act(SEA_VIEW *view)
   
   /* DISPLAY INTO NEW WINDOW */
   char *p;
-  h = 18;
   Fl_Window *win = new Fl_Window(600, 500, "Statistics");
-  win->callback(close_win_callback, NULL);
   Fl_Multiline_Output *ml = new Fl_Multiline_Output(2, 2, win->w()-4, win->h()-24);
   ml->textfont(FL_COURIER);
   ml->user_data(view->masename);
@@ -567,7 +557,6 @@ void geni_act(SEA_VIEW *view)
   Fl_Button *b = new Fl_Button(win->w()/2 - 25 , win->h() - 23, 50, 20, "Save");
   b->callback(save_stat_cb, ml);
   Fl_Return_Button *ok = new Fl_Return_Button(win->w() - 55 , win->h() - 23, 50, 20, "OK");
-  ok->callback(close_win_callback);
 
   if(view->tot_sel_seqs == 0) sprintf(text, "%d species\n", nb);
   else {
@@ -659,7 +648,7 @@ void geni_act(SEA_VIEW *view)
       sprintf(p, "Minimum : %.3f (%s - %s, %d transitions, %d transversions)\n", titvmin, titvminname1, titvminname2, (int)Pmin, (int)Qmin);
       p += strlen(p);
       sprintf(p, "Maximum : %.3f (%s - %s, %d transitions, %d transversions)\n", titvmax, titvmaxname1, titvmaxname2, (int)Pmax, (int)Qmax);
-      p += strlen(p);
+      //p += strlen(p);
     }
     else {
       sprintf(p, "Observed nucleotide pairs :\n"); p += strlen(p);
@@ -678,6 +667,14 @@ void geni_act(SEA_VIEW *view)
   ml->value(text);
   win->end();
   win->show();
+  while (TRUE) {
+    Fl_Widget* obj = Fl::readqueue();
+    if (!obj) Fl::wait();
+    else if (obj == ok || obj == win) {
+      delete win;
+      break;
+    }
+  }
   
   free(selected); free(text);
   for(i=0;i<nb;i++) {free(genseq[i]); free(comp[i]);}
