@@ -26,7 +26,7 @@ Fl_SVG_Graphics_Driver::Fl_SVG_Graphics_Driver(FILE *f) {
   out_ = f;
   width_ = 1;
   linecap_ = "butt";
-  family_ = strdup("");
+  family_ = "";
   bold_ = "";
   style_ = "";
   red_ = green_ = blue_ = 0;
@@ -34,7 +34,6 @@ Fl_SVG_Graphics_Driver::Fl_SVG_Graphics_Driver(FILE *f) {
 
 Fl_SVG_Graphics_Driver::~Fl_SVG_Graphics_Driver()
 {
-  free(family_);
 }
 
 void Fl_SVG_Graphics_Driver::rect(int x, int y, int w, int h) {
@@ -56,20 +55,16 @@ void Fl_SVG_Graphics_Driver::line(int x1, int y1, int x2, int y2) {
 
 void Fl_SVG_Graphics_Driver::font_(int ft, int s) {
   Fl_Graphics_Driver::font(ft, s);
-  Fl_Font f = Fl_Graphics_Driver::font();
-  int attr; char *p, *name;
-  name = strdup(Fl::get_font_name(f, &attr));
-  if ((p=strstr(name, " Bold")) || (p=strstr(name, " bold"))) memset(p+1, ' ', 4);
-  if ((p=strstr(name, " Italic")) || (p=strstr(name, " italic"))) memset(p+1, ' ', 6);
-  bold_ =  ( attr&FL_BOLD ? " font-weight=\"bold\"" : "" );
-  style_ =  ( attr&FL_ITALIC ? " font-style=\"italic\"" : "" );
-  if ((p=strstr(name, " Oblique")) || (p=strstr(name, " oblique"))) {
-     memset(p+1, ' ', 7);
-    style_ = " font-style=\"oblique\"";
-  }
-  trim_key(name);
-  free(family_);
-  family_ = name;
+  int famnum = ft/4;
+  if (famnum == 0) family_ = "Helvetica";
+  else if (famnum == 1) family_ = "Courier";
+  else family_ = "Times";
+  int modulo = ft % 4;
+  int use_bold = modulo == 1 || modulo == 3;
+  int use_italic = modulo >= 2;  
+  bold_ =  ( use_bold ? " font-weight=\"bold\"" : "" );
+  style_ =  ( use_italic ? " font-style=\"italic\"" : "" );
+  if (use_italic && famnum != 2) style_ = " font-style=\"oblique\"";
 }
 
 void Fl_SVG_Graphics_Driver::font(int ft, int s) {
@@ -86,19 +81,22 @@ void Fl_SVG_Graphics_Driver::line_style(int style, int width, char *dashes) {
 }
 
 void Fl_SVG_Graphics_Driver::draw(const char *str, int n, int x, int y) {
-  fprintf(out_, "<text x=\"%d\" y=\"%d\" font-family=\"%s\"%s%s font-size=\"%d\" xml:space=\"preserve\" "
-	  "fill=\"rgb(%u,%u,%u)\">%.*s</text>\n",x, y, family_, bold_, style_, size(), red_, green_, blue_, n, str);
+  // Caution: Internet Explorer ignores the xml:space="preserve" attribute
+  // work-around: replace all spaces by no-break space = U+00A0 = 0xC2-0xA0 (UTF-8) before sending to IE
+  fprintf(out_, "<text x=\"%d\" y=\"%d\" font-family=\"%s\"%s%s font-size=\"%d\" "
+	  "xml:space=\"preserve\" "
+	  " fill=\"rgb(%u,%u,%u)\">%.*s</text>\n",x, y, family_, bold_, style_, size(), red_, green_, blue_, n, str);
 }
 
- void Fl_SVG_Graphics_Driver::draw(const char* str, int n, float fx, float fy) {
-   return draw(str, n, (int)fx, (int)fy);
- }
+void Fl_SVG_Graphics_Driver::draw(const char* str, int n, float fx, float fy) {
+  return draw(str, n, (int)fx, (int)fy);
+}
 
- void Fl_SVG_Graphics_Driver::draw(int angle, const char* str, int n, int x, int y) {
-   fprintf(out_, "<g transform=\"translate(%d,%d) rotate(%d)\">", x, y, -angle);
-   draw(str, n, 0, 0);
-   fputs("</g>\n", out_);
- }
+void Fl_SVG_Graphics_Driver::draw(int angle, const char* str, int n, int x, int y) {
+  fprintf(out_, "<g transform=\"translate(%d,%d) rotate(%d)\">", x, y, -angle);
+  draw(str, n, 0, 0);
+  fputs("</g>\n", out_);
+}
  
 void Fl_SVG_Graphics_Driver::rtl_draw(const char*, int, int, int) {}
 
@@ -131,37 +129,35 @@ int Fl_SVG_Graphics_Driver::descent() {
 }
 
 #ifndef NO_PDF
-/* Fl_SVG_PDF_width_Graphics_Driver uses a PDF graphics driver only to compute string widths and heights,
- to allow creation of SVG data without calling fl_open_display().
- */
-class Fl_SVG_PDF_width_Graphics_Driver : public Fl_SVG_Graphics_Driver {
-  Fl_PDF_Graphics_Driver *pdf_driver;
-public:
-  Fl_SVG_PDF_width_Graphics_Driver(FILE *f) : Fl_SVG_Graphics_Driver(f), pdf_driver(new Fl_PDF_Graphics_Driver) {
+Fl_SVG_PDF_width_Graphics_Driver::Fl_SVG_PDF_width_Graphics_Driver(FILE *f) : Fl_SVG_Graphics_Driver(f), pdf_driver(new Fl_PDF_Graphics_Driver) {
     pdf_driver->pdf = PDF_new();
     PDF_begin_document(pdf_driver->pdf, "", 0, "");
     PDF_begin_page_ext(pdf_driver->pdf, 100, 100, "");
-  }
-  ~Fl_SVG_PDF_width_Graphics_Driver() {
+}
+
+Fl_SVG_PDF_width_Graphics_Driver::~Fl_SVG_PDF_width_Graphics_Driver() {
     PDF_end_page_ext(pdf_driver->pdf, "");
     PDF_end_document(pdf_driver->pdf, "");
     PDF_delete(pdf_driver->pdf);
     delete pdf_driver;
-    }
-  int descent() {
+}
+
+int Fl_SVG_PDF_width_Graphics_Driver::descent() {
     return pdf_driver->descent();
-  }
-  int height() {
+}
+
+int Fl_SVG_PDF_width_Graphics_Driver::height() {
     return pdf_driver->height();
-  }
-  double width(const char* str, int l) {
+}
+
+double Fl_SVG_PDF_width_Graphics_Driver::width(const char* str, int l) {
     return pdf_driver->width(str, l);
-  }
-  void font(int ft, int s) {
+}
+
+void Fl_SVG_PDF_width_Graphics_Driver::font(int ft, int s) {
     font_(ft, s);
     pdf_driver->font(ft, s);
-  }
-};
+}
 #endif // NO_PDF
 
 Fl_SVG_File_Surface::Fl_SVG_File_Surface(int w, int h, FILE *f, bool use_pdf_width) : Fl_Surface_Device(NULL) {
