@@ -3,6 +3,9 @@
 
 #include <stdio.h>
 #include <FL/Fl_Device.H>
+#ifndef NO_PDF
+#include "pdf_or_ps.h"
+#endif
 
 class Fl_SVG_Graphics_Driver : public Fl_Graphics_Driver {
   FILE *out_;
@@ -14,7 +17,7 @@ public:
   ~Fl_SVG_Graphics_Driver();
   FILE* file() {return out_;}
 protected:
-  char *family_;
+  const char *family_;
   const char *bold_;
   const char *style_;
   void rect(int x, int y, int w, int h);
@@ -53,5 +56,21 @@ public:
   ~Fl_SVG_File_Surface();
 };
 
+#ifndef NO_PDF
+/* Fl_SVG_PDF_width_Graphics_Driver uses a PDF graphics driver only to compute string widths and heights,
+ to allow creation of SVG data without calling fl_open_display().
+ */
+class Fl_SVG_PDF_width_Graphics_Driver : public Fl_SVG_Graphics_Driver {
+  Fl_PDF_Graphics_Driver *pdf_driver;
+public:
+  Fl_SVG_PDF_width_Graphics_Driver(FILE *f);
+  ~Fl_SVG_PDF_width_Graphics_Driver();
+  int descent();
+  int height();
+  double width(const char* str, int l);
+  void font(int ft, int s);
+};
+#endif  // ! NO_PDF
+
 #endif // FL_SVG_GRAPHICS_DRIVER_H
 
