@@ -2,11 +2,12 @@
 #define TREEDRAW_H
 
 #include "seaview.h"
+#include <FL/Fl_Window.H>
 #include <FL/Fl_Round_Button.H>
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Scrollbar.H>
-#include <FL/Fl_Counter.H>
+#include <FL/Fl_Simple_Counter.H>
 #define myrint(x) (int)floor(x + 0.5)
 
 
@@ -46,6 +47,9 @@ public:
 struct FD_nj_plot {
 	char *tree_name;
 	char *tree_label;
+  char *trees; // a Newick string with possibly several trees in it
+  int rank; // rank in trees of current_tree (from 0)
+  char *current_tree; // start of current tree within ->trees string
 	int notu;
 	int totbranches;
 	int has_br_length;
@@ -83,7 +87,6 @@ struct FD_nj_plot {
 	int font_family;
 	int swap;
 	float bootstrap_threshold;
-	char *current_tree;
 	SEA_VIEW *view;
 	Fl_Round_Button *full;
 	Fl_Round_Button *swap_button;
@@ -105,11 +108,15 @@ struct FD_nj_plot {
 	int edit_shape_rank;
 	int center_rank;
 	Fl_Button *select_clade_button, *delete_clade_button, *complete_edit_button;
-	Fl_Counter *zoom;
+	Fl_Simple_Counter *zoom;
+  Fl_Simple_Counter *change_tree;
 	Fl_Choice *root_unroot;
 	int page_count;
   bool branch_width_as_support;
   float support_threshold_high;
   float support_threshold_low;
 };
+
+extern Fl_Window *treedraw(char *tree, SEA_VIEW *view, const char *name, int from_tree_menu, int count = 1);
+
 #endif // TREEDRAW_H
