@@ -209,10 +209,6 @@ void trees_callback(Fl_Widget *ob, void *data)
 		}
 }
 
-/*static void interrupt_callback(Fl_Widget *wgt, void *data)
-{
-	*(int *)data = TRUE;
-}*/
 
 static void distance_callback(Fl_Widget *wgt, void *data)
 {
@@ -314,7 +310,7 @@ void distance_method_dialog(SEA_VIEW *view)
 	}
 	distance->value( FL_min(distance_choice, maxchoice) );
 	distance->callback(distance_callback, ignore_all_gaps);
-  keep_b_trees->value(0);
+        keep_b_trees->value(0);
 	treechoice->clear();
 	vlength_menu *vmenu = view->menu_trees;
 	int count = view->menu_trees->vlength();
@@ -322,6 +318,7 @@ void distance_method_dialog(SEA_VIEW *view)
 	  treechoice->add( vmenu->vitem(i)->label() );
 	treechoice->value(0);
 	treechoice->deactivate(); 
+        bootstrap->activate();
 	if (usertree->value()) bionj->setonly();
 	if (count) usertree->activate(); 
 	else usertree->deactivate(); 
@@ -2676,6 +2673,26 @@ void command_line_phylogeny(int argc, char **argv)
   }
   
   replicates = argval(argc, argv, "-replicates", 0);
+  
+  char *utree = NULL;
+  if (isarg(argc, argv, "-usertree")) {
+    char *utname;
+    if ( (utname = argname(argc, argv, "-usertree"))) {
+      FILE *in = fopen(utname, "r");
+      if (!in) {
+        fprintf(stderr, "Usertree file not found: %s\n",utname);
+        exit(1);
+      }
+      fseek(in, 0, SEEK_END);
+      long s = ftell(in);
+      fseek(in, 0, SEEK_SET);
+      utree = (char*)malloc(s+1);
+      fread(utree, s, 1, in);
+      utree[s] = 0;
+      fclose(in);
+    }
+  }
+
   if (isarg(argc, argv, "-distance")) { // distance method
     int distkind = 0; // == observed_p/ndist
     char *distname;
@@ -2690,11 +2707,13 @@ void command_line_phylogeny(int argc, char **argv)
       else if (strcmp(distname, "Kimura") == 0) distkind = Kimura_pdist;
       }
     if (distkind == 0) distname = strdup("Observed");
+    
     view->tot_trees = 0;
     tree = run_distance_method(view, distkind, isarg(argc, argv, "-nogaps"), !isarg(argc, argv, "-NJ"), replicates > 1, replicates, NULL,
-			       distname, distkind == Ka || distkind == Ks, argname(argc, argv, "-distance_matrix"), NULL, true, Fl::fatal, false);
+			       distname, distkind == Ka || distkind == Ks, argname(argc, argv, "-distance_matrix"), utree, true, Fl::fatal, false);
     if (isarg(argc, argv, "-distance_matrix")) exit(0);
   }
+  
   if (isarg(argc, argv, "-parsimony")) { // parsimony method
     const char *progname;
     jumbles = argval(argc, argv, "-jumbles", 0);
@@ -2703,9 +2722,16 @@ void command_line_phylogeny(int argc, char **argv)
     if (search && strcmp(search, "less") == 0) s_option = less_thorough;
     else if (search && strcmp(search, "best") == 0) s_option = rearrange_best_tree;
     else s_option = more_thorough;
+    int usertreerank = 0;
+    if (utree) {
+      view->tot_trees = 1;
+      view->trees = (char**)malloc(sizeof(char*));
+      view->trees[0] = utree;
+      usertreerank = 1;
+    }
     tree = run_parsimony_method(view, isarg(argc, argv, "-nogaps"), 
 				isarg(argc, argv, "-gaps_as_unknown") ? ", gaps treated as ?" : "", 
-				replicates, jumbles, 10000, 0, NULL, s_option, &progname, 1, true, 0, Fl::fatal);
+				replicates, jumbles, 10000, usertreerank, NULL, s_option, &progname, 1, true, 0, Fl::fatal);
     if (tree && (p = strchr(tree, ';')) != NULL) *(p+1) = 0; // tree string can contain several trees, keep first only
   }
   if (tree) {
