@@ -72,6 +72,7 @@ extern int calc_brl_for_lengthless(struct noeud *centre, struct noeud *pere);
 extern const char *preptree(FD_nj_plot *fd_nj_plot);
 extern void free_tree(FD_nj_plot *fd_nj_plot);
 extern char *ecrit_arbre_parenth(FD_nj_plot *fd_nj_plot, struct noeud *root);
+extern void replace_with_new_tree(FD_nj_plot *fd_nj_plot, char *newtree);
 
 
 /* globals */
@@ -87,14 +88,14 @@ void rooted_unrooted_callback(Fl_Widget *o, void *data)
 	if(fd_nj_plot->fd_unrooted != NULL) circular2squared(fd_nj_plot);
 	if(fd_nj_plot->lost_br_lengths) {
 	  fd_nj_plot->lost_br_lengths = FALSE;
-	  keep = fd_nj_plot->current_tree;
+    keep = fd_nj_plot->trees;
 	  keepn = fd_nj_plot->tree_name;
 	  keepl = fd_nj_plot->tree_label;
-	  fd_nj_plot->current_tree = NULL;
+	  fd_nj_plot->trees = NULL;
 	  fd_nj_plot->tree_name = NULL;
 	  fd_nj_plot->tree_label = NULL;
 	  free_tree(fd_nj_plot);
-	  fd_nj_plot->current_tree = keep;
+    fd_nj_plot->trees = keep;
 	  fd_nj_plot->tree_name = keepn;
 	  fd_nj_plot->tree_label = keepl;
 	  preptree(fd_nj_plot);//reinterpret tree for its branch lengths
@@ -107,16 +108,20 @@ void rooted_unrooted_callback(Fl_Widget *o, void *data)
 	fd_nj_plot->panel->window()->redraw();
   }
   else if(((Fl_Choice *)o)->value() == 1) {//goto circular display
+    if (fd_nj_plot->notu < 3) {
+      fd_nj_plot->root_unroot->value(0);
+      return;
+    }
 	if(fd_nj_plot->lost_br_lengths) {
 	  fd_nj_plot->lost_br_lengths = FALSE;
-	  keep = fd_nj_plot->current_tree;
+	  keep = fd_nj_plot->trees;
 	  keepn = fd_nj_plot->tree_name;
 	  keepl = fd_nj_plot->tree_label;
-	  fd_nj_plot->current_tree = NULL;
+	  fd_nj_plot->trees = NULL;
 	  fd_nj_plot->tree_name = NULL;
 	  fd_nj_plot->tree_label = NULL;
 	  free_tree(fd_nj_plot);
-	  fd_nj_plot->current_tree = keep;
+	  fd_nj_plot->trees = keep;
 	  fd_nj_plot->tree_name = keepn;
 	  fd_nj_plot->tree_label = keepl;
 	  preptree(fd_nj_plot);//reinterpret tree for its branch lengths
@@ -127,21 +132,20 @@ void rooted_unrooted_callback(Fl_Widget *o, void *data)
 	if(fd_nj_plot->fd_unrooted == NULL) squared2circular(fd_nj_plot, (Fl_Choice *)o);
 	}
   else if(((Fl_Choice *)o)->value() == 2) {//goto Cladogram display
-	if(fd_nj_plot->fd_unrooted != NULL) circular2squared(fd_nj_plot);
-	keep = ecrit_arbre_parenth(fd_nj_plot, fd_nj_plot->racine);
-	free(fd_nj_plot->current_tree);
-	fd_nj_plot->current_tree = keep;
-	fd_nj_plot->lost_br_lengths = TRUE;
-	fd_nj_plot->has_br_length = FALSE;
-	calc_brl_for_lengthless(fd_nj_plot->racine, NULL);
-	fd_nj_plot->need_runtree = TRUE;
-	fd_nj_plot->l_button->deactivate();
-	fd_nj_plot->new_outgroup->deactivate();
-	fd_nj_plot->swap_button->deactivate();
-	fd_nj_plot->choix = show_tree;
-	fd_nj_plot->full->setonly();
-	fd_nj_plot->panel->show();
-	fd_nj_plot->panel->window()->redraw();
+    if(fd_nj_plot->fd_unrooted != NULL) circular2squared(fd_nj_plot);
+    keep = ecrit_arbre_parenth(fd_nj_plot, fd_nj_plot->racine); // to share reroot/swaps between squared/cladogram
+    replace_with_new_tree(fd_nj_plot, keep);
+    fd_nj_plot->lost_br_lengths = TRUE;
+    fd_nj_plot->has_br_length = FALSE;
+    calc_brl_for_lengthless(fd_nj_plot->racine, NULL);
+    fd_nj_plot->need_runtree = TRUE;
+    fd_nj_plot->l_button->deactivate();
+    fd_nj_plot->new_outgroup->deactivate();
+    fd_nj_plot->swap_button->deactivate();
+    fd_nj_plot->choix = show_tree;
+    fd_nj_plot->full->setonly();
+    fd_nj_plot->panel->show();
+    fd_nj_plot->panel->window()->redraw();
 	}
 }
 
