@@ -93,6 +93,7 @@ extern color_choice prep_aa_color_code(char *list_std, char *list_alt,
 extern void load_resources(const char *progname);
 extern void allonge_seqs(char **seq, int totseqs, int maxlen, int *eachlength,
 			 int tot_comment_lines, char **comment_line, char **pregion_line);
+extern int int_res_value(const char *name, int def_value);
 extern const char *progname;
 #if defined(__APPLE__)
 extern const char *MG_GetBundleResourcesDir(void);
@@ -119,6 +120,7 @@ extern int get_ncbi_gc_from_comment(char *comment);
 extern int create_gblocks_mask(SEA_VIEW *view, region *myregion, int no_gui, int b5_val, int b4_val, int b3_val, int b2_val);
 extern char *back_translate_with_gaps(char *prot, char *dna);
 extern void del_gap_only_sites(SEA_VIEW *view);
+extern void save_bootstrap_replicates(const char *fname, int replicates, SEA_VIEW *view);
 extern "C" {
   int get_acnuc_gc_number(int ncbi_gc);
   int get_ncbi_gc_number(int ncbi_gc);
@@ -1014,14 +1016,18 @@ if(numb_species_sets != 0) { /* sauver les species sets */
 if(tot_trees > 0) { /* write trees out */
 	for(num = 0; num < tot_trees; num++) {
 		fprintf(out, ";;$ %s\n", menu_tree_items[num].label());
-		int l = strlen(trees[num]);
-		for(char *p = trees[num]; p < trees[num] + l; p += 80) {
+    char *tree = strdup(trees[num]);
+    char *p;
+    while ((p=strchr(tree, '\n')) != NULL) *p = ' ';
+    int l = strlen(tree);
+		for (p = tree; p < tree + l; p += 80) {
 			while (*p == '$') { // avoid begin line with ";;$..." that marks a new tree
 			  fputs(";;$\n", out);
 			  p++;
 			  }
 			fprintf(out, ";;%.80s\n", p);
 			}
+    free(tree);
 		}
 	}
 for(num=0; num<totseqs; num++) {
@@ -2146,6 +2152,7 @@ void format_conversion(int argc, char **argv)
   int i, j, gc;
   bool std_output = false;
   bool save_fragment = false;
+  bool bootstrap = false;
   region *myregion;
   
   SEA_VIEW *view = cmdline_read_input_alignment(argc, argv);
@@ -2154,6 +2161,8 @@ void format_conversion(int argc, char **argv)
     exit(1);
   }  
   known_format out_format = view->format_for_save;
+  load_resources(progname);
+  view->phylipwidnames = int_res_value("phylipwidnames", 30);
   char *outfile = process_output_options(argc, argv, out_format, std_output);
   
   if ( !view->protein && isarg(argc, argv, "-translate")) {
@@ -2164,9 +2173,9 @@ void format_conversion(int argc, char **argv)
       free(view->sequence[i]);
       view->sequence[i] = p;
       if (no_terminal_stop) {
-	char *q = p + strlen(p) - 1;
-	while (q > p && *q == '-') q--;
-	if (q >= p && *q == '*') *q = '-';
+        char *q = p + strlen(p) - 1;
+        while (q > p && *q == '-') q--;
+        if (q >= p && *q == '*') *q = '-';
       }
       view->each_length[i] = strlen(p);
       view->regions = NULL;
@@ -2298,6 +2307,16 @@ void format_conversion(int argc, char **argv)
   }
   if (isarg(argc, argv, "-del_gap_only_sites") && !isarg(argc, argv, "-sites")) del_gap_only_sites(view);
   
+  if ( (p = argname(argc, argv, "-bootstrap")) ) {
+    int replicates = -1;
+    sscanf(p, "%d", &replicates);
+    if (replicates == -1) replicates = 1;
+    bootstrap = true;
+    save_fragment = false;
+    save_bootstrap_replicates(outfile, replicates, view);
+  }
+
+  
   if (save_fragment) {
     myregion = view->active_region;
     if (myregion == NULL) {
@@ -2316,7 +2335,7 @@ void format_conversion(int argc, char **argv)
 					   view->tot_trees, view->trees, 
 					   view->menu_trees->vitem(0), view->spaces_in_fasta_names);
   }
-  else {
+  else if (!bootstrap) {
     err_message = save_alignment_or_region(outfile, view->sequence, view->comments,
 				 view->header, view->seqname, view->tot_seqs, view->each_length,
 				 view->regions, NULL, out_format,
@@ -2340,7 +2359,7 @@ void format_conversion(int argc, char **argv)
     }
     fclose(in);
     fflush(stdout);
-    delete_tmp_filename(outfile);
+    fl_unlink(outfile);
     }
   exit(0);
 }
@@ -2378,7 +2397,7 @@ char *process_output_options(int argc, char **argv, known_format& out_format, bo
     }
   }
   else if (strcmp(outfile, "-") == 0) {
-    outfile = create_tmp_filename();
+    outfile = strdup(create_tmp_filename());
     std_output = true;
   }
   return outfile;
