@@ -166,11 +166,11 @@ char *translate_with_gaps(char *seq, int gc)
 
 char *back_translate_with_gaps(char *prot, char *dna)
 {
-	int l;
-	char *newdna, *pd;
+	int l, ld;
+	char *newdna, *pd, *q;
 	
-	l = strlen(prot);
-	newdna = (char *)malloc(3*l + 7);
+	l = 3*strlen(prot) + 7;
+	newdna = (char *)malloc(l);
 	if(newdna == NULL) return NULL;
 	pd = newdna;
 	prot--;
@@ -191,10 +191,18 @@ char *back_translate_with_gaps(char *prot, char *dna)
 		}
 	*pd = 0;
 	while(memcmp(dna, "---", 3) == 0) dna += 3;
-	if(*dna != 0) {//if a partial codon remains at end
-	  while(*(pd-1) == '-') pd--;
-	  strcpy(pd, dna);
+	if (*dna != 0) {//if a partial codon remains at end
+	  while (*(pd-1) == '-') pd--;
+	  ld = pd - newdna;
+	  q = dna + strlen(dna);
+	  while (q > dna && *(q-1) == '-') q--;
+	  if (ld+(q-dna) >= l) {
+	    l = ld+(q-dna)+5;
+	    newdna = (char*)realloc(newdna, l);
+	    pd = newdna + ld;
 	  }
+	  memcpy(pd, dna, q - dna); pd += q - dna; *pd = 0;
+	}
 	return newdna;
 }
 
