@@ -4,7 +4,9 @@
 #include <FL/fl_utf8.h>
 #include <stdlib.h>
 #include <stdio.h>
+#ifndef NO_PDF
 #include <pdflib.h>
+#endif
 
 /* included functions */
 void my_bell(void);
@@ -67,6 +69,7 @@ char *get_prog_dir(void)
 		WCHAR *wp = GetCommandLineW();
 		l = 4*wcslen(wp)+1;
 		line = new char[l];
+    char *keep = line;
 		fl_utf8fromwc(line, l, wp, wcslen(wp));
 		//fprintf(OUT,"retour GetCommandLine line=%s\n",line);fflush(OUT);
 		/* extract 1st word of line or first group delimited by " " */
@@ -80,7 +83,7 @@ char *get_prog_dir(void)
 			dir = (char *)malloc(l + 1);
 			memcpy(dir, line, l); dir[l] = 0;
 		}
-		delete[] line;
+		delete[] keep;
 	}
 	return dir;
 }
@@ -135,6 +138,7 @@ const char *win32_UTF8_to_CP1252(const char *in)
 }
 
 
+#ifndef NO_PDF
 static size_t pdf_write_proc(PDF *p, void *data, size_t size)
 {
   FILE **output = (FILE**)PDF_get_opaque(p);
@@ -177,6 +181,7 @@ void win32_PDF_delete(PDF *p)
   delete output;
   PDF_delete(p);
 }
+#endif
 
 /* to delete all "base_fname*" files
  */
