@@ -90,12 +90,12 @@ public:
 
 void my_watch_cursor(Fl_Window *win)
 {
-win->cursor(FL_CURSOR_WAIT); 
+win->default_cursor(FL_CURSOR_WAIT); 
 }
 
 void fl_reset_cursor(Fl_Window *win)
 {
-win->cursor(FL_CURSOR_DEFAULT); 
+win->default_cursor(FL_CURSOR_DEFAULT); 
 }
 
 
